@@ -78,8 +78,12 @@ def shapes(source, mask=None, connectivity=4, transform=IDENTITY):
         Data type must be one of rasterio.int16, rasterio.int32,
         rasterio.uint8, rasterio.uint16, or rasterio.float32.
     mask : numpy ndarray or rasterio Band object, optional
-        Values of False or 0 will be excluded from feature generation
-        Must evaluate to bool (rasterio.bool_ or rasterio.uint8)
+        Must evaluate to bool (rasterio.bool_ or rasterio.uint8). Values
+        of False or 0 will be excluded from feature generation.  Note
+        well that this is the inverse sense from Numpy's, where a mask
+        value of True indicates invalid data in an array. If `source` is
+        a Numpy masked array and `mask` is None, the source's mask will
+        be inverted and used in place of `mask`.
     connectivity : int, optional
         Use 4 or 8 pixel connectivity for grouping pixels into features
     transform : Affine transformation, optional
@@ -104,6 +108,10 @@ def shapes(source, mask=None, connectivity=4, transform=IDENTITY):
     memory.
 
     """
+    if hasattr(source, 'mask') and mask is None:
+        mask = ~source.mask
+        source = source.data
+
     transform = guard_transform(transform)
     for s, v in _shapes(source, mask, connectivity, transform):
         yield s, v
@@ -169,9 +177,11 @@ def rasterize(
 
     Parameters
     ----------
-    shapes : iterable of (geometry, value) pairs or iterable over
-        geometries. `geometry` can either be an object that implements
-        the geo interface or GeoJSON-like object.
+    shapes : iterable of (`geometry`, `value`) pairs or iterable over
+        geometries. The `geometry` can either be an object that
+        implements the geo interface or GeoJSON-like object. If no
+        `value` is provided the `default_value` will be used. If `value`
+        is `None` the `fill` value will be used.
     out_shape : tuple or list with 2 integers
         Shape of output numpy ndarray.
     fill : int or float, optional
@@ -189,10 +199,11 @@ def rasterize(
         false, only pixels whose center is within the polygon or that
         are selected by Bresenham's line algorithm will be burned in.
     merge_alg : MergeAlg, optional
-        Merge algorithm to use.  One of:
+        Merge algorithm to use. One of:
             MergeAlg.replace (default): the new value will overwrite the
-                existing value.
-            MergeAlg.add: the new value will be added to the existing raster.
+              existing value.
+            MergeAlg.add: the new value will be added
+              to the existing raster.
     default_value : int or float, optional
         Used as value for all geometries, if not provided in `shapes`.
     dtype : rasterio or numpy data type, optional
@@ -247,6 +258,8 @@ def rasterize(
     for index, item in enumerate(shapes):
         if isinstance(item, (tuple, list)):
             geom, value = item
+            if value is None:
+                value = fill
         else:
             geom = item
             value = default_value
