@@ -600,3 +600,11 @@ requires_gdal21 = pytest.mark.skipif(
 requires_gdal22 = pytest.mark.skipif(
     not gdal_version.at_least('2.2'),
     reason="Requires GDAL 2.2.x")
+
+requires_gdal_lt_3 = pytest.mark.skipif(
+    gdal_version.__lt__('3.0'),
+    reason="Requires GDAL 1.x/2.x")
+
+requires_gdal3 = pytest.mark.skipif(
+    not gdal_version.at_least('3.0'),
+    reason="Requires GDAL 3.0.x")
