@@ -15,7 +15,7 @@ from rasterio.crs import CRS
 from rasterio.env import env_ctx_if_needed, Env
 from rasterio.errors import CRSError
 
-from .conftest import requires_gdal21, requires_gdal22
+from .conftest import requires_gdal21, requires_gdal22, requires_gdal_lt_3
 
 
 # Items like "D_North_American_1983" characterize the Esri dialect
@@ -72,6 +72,7 @@ def test_read_epsg():
         assert src.crs.to_epsg() == 32618
 
 
+@requires_gdal_lt_3
 def test_read_esri_wkt():
     with rasterio.open('tests/data/test_esri_wkt.tif') as src:
         assert 'PROJCS["USA_Contiguous_Albers_Equal_Area_Conic_USGS_version",' in src.crs.wkt
@@ -110,7 +111,7 @@ def test_write_3857(tmpdir):
     info = subprocess.check_output([
         'gdalinfo', dst_path])
     # WKT string may vary a bit w.r.t GDAL versions
-    assert 'PROJCS["WGS 84 / Pseudo-Mercator"' in info.decode('utf-8')
+    assert '"WGS 84 / Pseudo-Mercator"' in info.decode('utf-8')
 
 
 def test_write_bogus_fails(tmpdir, profile_rgb_byte_tif):
@@ -260,7 +261,7 @@ def test_has_wkt_property():
 
 
 def test_repr():
-    assert repr(CRS({'init': 'epsg:4326'})).startswith("CRS.from_dict(init")
+    assert repr(CRS({'init': 'epsg:4326'})).startswith("CRS.from_epsg(4326)")
 
 
 def test_dunder_str():
@@ -326,6 +327,7 @@ def test_from_user_input_epsg():
     assert 'init' in CRS.from_user_input('epsg:4326')
 
 
+@requires_gdal_lt_3
 @pytest.mark.parametrize('projection_string', [ESRI_PROJECTION_STRING])
 def test_from_esri_wkt_no_fix(projection_string):
     """Test ESRI CRS morphing with no datum fixing"""
@@ -334,6 +336,7 @@ def test_from_esri_wkt_no_fix(projection_string):
         assert 'DATUM["D_North_American_1983"' in crs.wkt
 
 
+@requires_gdal_lt_3
 @pytest.mark.parametrize('projection_string', [ESRI_PROJECTION_STRING])
 def test_from_esri_wkt_fix_datum(projection_string):
     """Test ESRI CRS morphing with datum fixing"""
@@ -342,6 +345,7 @@ def test_from_esri_wkt_fix_datum(projection_string):
         assert 'DATUM["North_American_Datum_1983"' in crs.wkt
 
 
+@requires_gdal_lt_3
 def test_to_esri_wkt_fix_datum():
     """Morph to Esri form"""
     assert 'DATUM["D_North_American_1983"' in CRS(init='epsg:26913').to_wkt(morph_to_esri_dialect=True)
@@ -410,6 +414,7 @@ def test_issue1609_wktext_a():
     assert 'PARAMETER["latitude_of_origin",-70]' in wkt
 
 
+@requires_gdal_lt_3
 def test_issue1609_wktext_b():
     """Check on fix of issue 1609"""
     dst_proj = {'ellps': 'WGS84',
@@ -451,3 +456,18 @@ def test_linear_units():
 def test_crs_copy():
     """CRS can be copied"""
     assert copy.copy(CRS.from_epsg(3857)).wkt.startswith('PROJCS["WGS 84 / Pseudo-Mercator",GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84"')
+
+
+def test_crs_hash():
+    """hashes of equivalent CRS are equal"""
+    assert hash(CRS.from_epsg(3857)) == hash(CRS.from_epsg(3857))
+
+
+def test_crs_hash_unequal():
+    """hashes of non-equivalent CRS are not equal"""
+    assert hash(CRS.from_epsg(3857)) != hash(CRS.from_epsg(4326))
+
+
+def test_crs84():
+    """Create CRS from OGC code"""
+    assert "WGS 84" in CRS.from_user_input("urn:ogc:def:crs:OGC::CRS84").wkt
